@@ -4,7 +4,7 @@ use std::fs;
 use std::io::{BufRead, ErrorKind, Write};
 use std::path::Path;
 use std::sync::Arc;
-use std::{cmp, env, iter};
+use std::{cmp, env};
 
 use anyhow::{anyhow, Context, Result};
 use git_testament::{git_testament, render_testament};
@@ -21,10 +21,10 @@ use crate::utils::notify::NotificationLevel;
 use crate::utils::utils;
 use crate::{Cfg, Notification, Toolchain, UpdateStatus};
 
-pub const WARN_COMPLETE_PROFILE: &str = "downloading with complete profile isn't recommended unless you are a developer of the rust language";
+pub(crate) const WARN_COMPLETE_PROFILE: &str = "downloading with complete profile isn't recommended unless you are a developer of the rust language";
 
-pub fn confirm(question: &str, default: bool) -> Result<bool> {
-    write!(process().stdout(), "{} ", question)?;
+pub(crate) fn confirm(question: &str, default: bool) -> Result<bool> {
+    write!(process().stdout(), "{question} ")?;
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
@@ -40,13 +40,13 @@ pub fn confirm(question: &str, default: bool) -> Result<bool> {
     Ok(r)
 }
 
-pub enum Confirm {
+pub(crate) enum Confirm {
     Yes,
     No,
     Advanced,
 }
 
-pub fn confirm_advanced() -> Result<Confirm> {
+pub(crate) fn confirm_advanced() -> Result<Confirm> {
     writeln!(process().stdout())?;
     writeln!(process().stdout(), "1) Proceed with installation (default)")?;
     writeln!(process().stdout(), "2) Customize installation")?;
@@ -67,8 +67,8 @@ pub fn confirm_advanced() -> Result<Confirm> {
     Ok(r)
 }
 
-pub fn question_str(question: &str, default: &str) -> Result<String> {
-    writeln!(process().stdout(), "{} [{}]", question, default)?;
+pub(crate) fn question_str(question: &str, default: &str) -> Result<String> {
+    writeln!(process().stdout(), "{question} [{default}]")?;
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
@@ -81,9 +81,9 @@ pub fn question_str(question: &str, default: &str) -> Result<String> {
     }
 }
 
-pub fn question_bool(question: &str, default: bool) -> Result<bool> {
+pub(crate) fn question_bool(question: &str, default: bool) -> Result<bool> {
     let default_text = if default { "(Y/n)" } else { "(y/N)" };
-    writeln!(process().stdout(), "{} {}", question, default_text)?;
+    writeln!(process().stdout(), "{question} {default_text}")?;
 
     let _ = std::io::stdout().flush();
     let input = read_line()?;
@@ -101,7 +101,7 @@ pub fn question_bool(question: &str, default: bool) -> Result<bool> {
     }
 }
 
-pub fn read_line() -> Result<String> {
+pub(crate) fn read_line() -> Result<String> {
     let stdin = process().stdin();
     let stdin = stdin.lock();
     let mut lines = stdin.lines();
@@ -132,7 +132,7 @@ impl NotifyOnConsole {
             }
         };
         let level = n.level();
-        for n in format!("{}", n).lines() {
+        for n in format!("{n}").lines() {
             match level {
                 NotificationLevel::Verbose => {
                     if self.verbose {
@@ -156,7 +156,7 @@ impl NotifyOnConsole {
     }
 }
 
-pub fn set_globals(verbose: bool, quiet: bool) -> Result<Cfg> {
+pub(crate) fn set_globals(verbose: bool, quiet: bool) -> Result<Cfg> {
     use std::cell::RefCell;
 
     use super::download_tracker::DownloadTracker;
@@ -175,7 +175,11 @@ pub fn set_globals(verbose: bool, quiet: bool) -> Result<Cfg> {
     }))
 }
 
-pub fn show_channel_update(cfg: &Cfg, name: &str, updated: Result<UpdateStatus>) -> Result<()> {
+pub(crate) fn show_channel_update(
+    cfg: &Cfg,
+    name: &str,
+    updated: Result<UpdateStatus>,
+) -> Result<()> {
     show_channel_updates(cfg, vec![(name.to_string(), updated)])
 }
 
@@ -229,18 +233,18 @@ fn show_channel_updates(cfg: &Cfg, toolchains: Vec<(String, Result<UpdateStatus>
 
     for (name, banner, width, color, version, previous_version) in data {
         let padding = max_width - width;
-        let padding: String = iter::repeat(' ').take(padding).collect();
-        let _ = write!(t, "  {}", padding);
+        let padding: String = " ".repeat(padding);
+        let _ = write!(t, "  {padding}");
         let _ = t.attr(term2::Attr::Bold);
         if let Some(color) = color {
             let _ = t.fg(color);
         }
-        let _ = write!(t, "{} ", name);
-        let _ = write!(t, "{}", banner);
+        let _ = write!(t, "{name} ");
+        let _ = write!(t, "{banner}");
         let _ = t.reset();
-        let _ = write!(t, " - {}", version);
+        let _ = write!(t, " - {version}");
         if let Some(previous_version) = previous_version {
-            let _ = write!(t, " (from {})", previous_version);
+            let _ = write!(t, " (from {previous_version})");
         }
         let _ = writeln!(t);
     }
@@ -249,7 +253,7 @@ fn show_channel_updates(cfg: &Cfg, toolchains: Vec<(String, Result<UpdateStatus>
     Ok(())
 }
 
-pub fn update_all_channels(
+pub(crate) fn update_all_channels(
     cfg: &Cfg,
     do_self_update: bool,
     force_update: bool,
@@ -277,34 +281,17 @@ pub fn update_all_channels(
 }
 
 #[derive(Clone, Copy, Debug)]
-pub enum SelfUpdatePermission {
+pub(crate) enum SelfUpdatePermission {
     HardFail,
     Skip,
     Permit,
 }
 
-pub fn self_update_permitted(explicit: bool) -> Result<SelfUpdatePermission> {
+pub(crate) fn self_update_permitted(explicit: bool) -> Result<SelfUpdatePermission> {
     if cfg!(windows) {
         Ok(SelfUpdatePermission::Permit)
     } else {
         // Detect if rustup is not meant to self-update
-        match process().var("SNAP") {
-            Ok(_) => {
-                // We're running under snappy so don't even bother
-                // trying to self-update
-                // TODO: Report this to the user?
-                // TODO: Maybe ask snapd if there's an update and report
-                //       that to the user instead?
-                debug!("Skipping self-update because SNAP was detected");
-                if explicit {
-                    return Ok(SelfUpdatePermission::HardFail);
-                } else {
-                    return Ok(SelfUpdatePermission::Skip);
-                }
-            }
-            Err(env::VarError::NotPresent) => {}
-            Err(e) => return Err(e).context("Could not interrogate SNAP environment variable"),
-        }
         let current_exe = env::current_exe()?;
         let current_exe_dir = current_exe.parent().expect("Rustup isn't in a directory‽");
         if let Err(e) = tempfile::Builder::new()
@@ -327,7 +314,7 @@ pub fn self_update_permitted(explicit: bool) -> Result<SelfUpdatePermission> {
     }
 }
 
-pub fn self_update<F>(before_restart: F) -> Result<utils::ExitCode>
+pub(crate) fn self_update<F>(before_restart: F) -> Result<utils::ExitCode>
 where
     F: FnOnce() -> Result<utils::ExitCode>,
 {
@@ -354,9 +341,9 @@ where
     Ok(utils::ExitCode(0))
 }
 
-pub fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
+pub(crate) fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
     let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(&toolchain)?;
+    let distributable = DistributableToolchain::new_for_components(toolchain)?;
     let components = distributable.list_components()?;
     for component in components {
         if component.component.short_name_in_manifest() == "rust-std" {
@@ -367,10 +354,10 @@ pub fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
                 .expect("rust-std should have a target");
             if component.installed {
                 let _ = t.attr(term2::Attr::Bold);
-                let _ = writeln!(t, "{} (installed)", target);
+                let _ = writeln!(t, "{target} (installed)");
                 let _ = t.reset();
             } else if component.available {
-                let _ = writeln!(t, "{}", target);
+                let _ = writeln!(t, "{target}");
             }
         }
     }
@@ -378,9 +365,9 @@ pub fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
     Ok(utils::ExitCode(0))
 }
 
-pub fn list_installed_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
+pub(crate) fn list_installed_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
     let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(&toolchain)?;
+    let distributable = DistributableToolchain::new_for_components(toolchain)?;
     let components = distributable.list_components()?;
     for component in components {
         if component.component.short_name_in_manifest() == "rust-std" {
@@ -390,34 +377,34 @@ pub fn list_installed_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCo
                 .as_ref()
                 .expect("rust-std should have a target");
             if component.installed {
-                writeln!(t, "{}", target)?;
+                writeln!(t, "{target}")?;
             }
         }
     }
     Ok(utils::ExitCode(0))
 }
 
-pub fn list_components(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
+pub(crate) fn list_components(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
     let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(&toolchain)?;
+    let distributable = DistributableToolchain::new_for_components(toolchain)?;
     let components = distributable.list_components()?;
     for component in components {
         let name = component.name;
         if component.installed {
             t.attr(term2::Attr::Bold)?;
-            writeln!(t, "{} (installed)", name)?;
+            writeln!(t, "{name} (installed)")?;
             t.reset()?;
         } else if component.available {
-            writeln!(t, "{}", name)?;
+            writeln!(t, "{name}")?;
         }
     }
 
     Ok(utils::ExitCode(0))
 }
 
-pub fn list_installed_components(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
+pub(crate) fn list_installed_components(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
     let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(&toolchain)?;
+    let distributable = DistributableToolchain::new_for_components(toolchain)?;
     let components = distributable.list_components()?;
     for component in components {
         if component.installed {
@@ -436,7 +423,7 @@ fn print_toolchain_path(
 ) -> Result<()> {
     let toolchain_path = {
         let mut t_path = cfg.toolchains_dir.clone();
-        t_path.push(&toolchain);
+        t_path.push(toolchain);
         t_path
     };
     let toolchain_meta = fs::symlink_metadata(&toolchain_path)?;
@@ -460,7 +447,7 @@ fn print_toolchain_path(
     Ok(())
 }
 
-pub fn list_toolchains(cfg: &Cfg, verbose: bool) -> Result<utils::ExitCode> {
+pub(crate) fn list_toolchains(cfg: &Cfg, verbose: bool) -> Result<utils::ExitCode> {
     let toolchains = cfg.list_toolchains()?;
     if toolchains.is_empty() {
         writeln!(process().stdout(), "no installed toolchains")?;
@@ -495,7 +482,7 @@ pub fn list_toolchains(cfg: &Cfg, verbose: bool) -> Result<utils::ExitCode> {
     Ok(utils::ExitCode(0))
 }
 
-pub fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
+pub(crate) fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
     let overrides = cfg.settings_file.with(|s| Ok(s.overrides.clone()))?;
 
     if overrides.is_empty() {
@@ -528,7 +515,7 @@ pub fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
 
 git_testament!(TESTAMENT);
 
-pub fn version() -> &'static str {
+pub(crate) fn version() -> &'static str {
     lazy_static! {
         // Because we trust our `stable` branch given the careful release
         // process, we mark it trusted here so that our version numbers look
@@ -538,7 +525,7 @@ pub fn version() -> &'static str {
     &RENDERED
 }
 
-pub fn dump_testament() -> Result<utils::ExitCode> {
+pub(crate) fn dump_testament() -> Result<utils::ExitCode> {
     use git_testament::GitModification::*;
     writeln!(
         process().stdout(),
@@ -618,7 +605,7 @@ pub fn report_error(e: &anyhow::Error) {
     }
 }
 
-pub fn ignorable_error(error: &'static str, no_prompt: bool) -> Result<()> {
+pub(crate) fn ignorable_error(error: &'static str, no_prompt: bool) -> Result<()> {
     let error = anyhow!(error);
     report_error(&error);
     if no_prompt {
