@@ -32,16 +32,16 @@
 
 #[cfg(unix)]
 mod shell;
-pub mod test;
+pub(crate) mod test;
 #[cfg(unix)]
 mod unix;
 #[cfg(windows)]
 mod windows;
 mod os {
     #[cfg(unix)]
-    pub use super::unix::*;
+    pub(crate) use super::unix::*;
     #[cfg(windows)]
-    pub use super::windows::*;
+    pub(crate) use super::windows::*;
 }
 
 use std::borrow::Cow;
@@ -57,7 +57,7 @@ use anyhow::{anyhow, Context, Result};
 use cfg_if::cfg_if;
 use same_file::Handle;
 
-use super::common::{self, ignorable_error, Confirm};
+use super::common::{self, ignorable_error, report_error, Confirm};
 use super::errors::*;
 use super::markdown::md;
 use super::term2;
@@ -70,7 +70,7 @@ use crate::utils::Notification;
 use crate::{Cfg, UpdateStatus};
 use crate::{DUP_TOOLS, TOOLS};
 use os::*;
-pub use os::{delete_rustup_and_cargo_home, run_update, self_replace};
+pub(crate) use os::{delete_rustup_and_cargo_home, run_update, self_replace};
 #[cfg(windows)]
 pub use windows::complete_windows_uninstall;
 
@@ -85,11 +85,11 @@ pub struct InstallOpts<'a> {
 }
 
 #[cfg(feature = "no-self-update")]
-pub const NEVER_SELF_UPDATE: bool = true;
+pub(crate) const NEVER_SELF_UPDATE: bool = true;
 #[cfg(not(feature = "no-self-update"))]
-pub const NEVER_SELF_UPDATE: bool = false;
+pub(crate) const NEVER_SELF_UPDATE: bool = false;
 
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub enum SelfUpdateMode {
     Enable,
     Disable,
@@ -97,11 +97,11 @@ pub enum SelfUpdateMode {
 }
 
 impl SelfUpdateMode {
-    pub fn modes() -> &'static [&'static str] {
+    pub(crate) fn modes() -> &'static [&'static str] {
         &["enable", "disable", "check-only"]
     }
 
-    pub fn default_mode() -> &'static str {
+    pub(crate) fn default_mode() -> &'static str {
         "enable"
     }
 }
@@ -153,7 +153,7 @@ home directory, located at:
 
 This can be modified with the RUSTUP_HOME environment variable.
 
-The Cargo home directory located at:
+The Cargo home directory is located at:
 
     {cargo_home}
 
@@ -209,15 +209,15 @@ but will not be added automatically."
 #[cfg(not(windows))]
 macro_rules! post_install_msg_unix {
     () => {
-        r"# Rust is installed now. Great!
+        r#"# Rust is installed now. Great!
 
 To get started you may need to restart your current shell.
 This would reload your `PATH` environment variable to include
 Cargo's bin directory ({cargo_home}/bin).
 
 To configure your current shell, run:
-    source {cargo_home}/env
-"
+    source "{cargo_home}/env"
+"#
     };
 }
 
@@ -237,14 +237,14 @@ Cargo's bin directory ({cargo_home}\\bin).
 #[cfg(not(windows))]
 macro_rules! post_install_msg_unix_no_modify_path {
     () => {
-        r"# Rust is installed now. Great!
+        r#"# Rust is installed now. Great!
 
 To get started you need Cargo's bin directory ({cargo_home}/bin) in your `PATH`
 environment variable. This has not been done automatically.
 
 To configure your current shell, run:
-    source {cargo_home}/env
-"
+    source "{cargo_home}/env"
+"#
     };
 }
 
@@ -276,21 +276,22 @@ static MSVC_MESSAGE: &str = r#"# Rust Visual C++ prerequisites
 Rust requires the Microsoft C++ build tools for Visual Studio 2013 or
 later, but they don't seem to be installed.
 
-The easiest way to acquire the build tools is by installing Microsoft
-Visual C++ Build Tools 2019 which provides just the Visual C++ build
-tools:
+"#;
 
-    https://visualstudio.microsoft.com/visual-cpp-build-tools/
-
-Please ensure the Windows 10 SDK and the English language pack components
-are included when installing the Visual C++ Build Tools.
-
-Alternately, you can install Visual Studio 2019, Visual Studio 2017,
-Visual Studio 2015, or Visual Studio 2013 and during install select
-the "C++ tools":
+#[cfg(windows)]
+static MSVC_MANUAL_INSTALL_MESSAGE: &str = r#"
+You can acquire the build tools by installing Microsoft Visual Studio.
 
     https://visualstudio.microsoft.com/downloads/
 
+Check the box for "Desktop development with C++" which will ensure that the
+needed components are installed. If your locale language is not English,
+then additionally check the box for English under Language packs.
+
+For more details see:
+
+    https://rust-lang.github.io/rustup/installation/windows-msvc.html
+
 _Install the C++ build tools before proceeding_.
 
 If you will be targeting the GNU ABI or otherwise know what you are
@@ -298,6 +299,15 @@ doing then it is fine to continue installation without the build
 tools, but otherwise, install the C++ build tools before proceeding.
 "#;
 
+#[cfg(windows)]
+static MSVC_AUTO_INSTALL_MESSAGE: &str = r#"# Rust Visual C++ prerequisites
+
+Rust requires a linker and Windows API libraries but they don't seem to be available.
+
+These components can be acquired through a Visual Studio installer.
+
+"#;
+
 static UPDATE_ROOT: &str = "https://static.rust-lang.org/rustup";
 
 /// `CARGO_HOME` suitable for display, possibly with $HOME
@@ -324,7 +334,7 @@ fn canonical_cargo_home() -> Result<Cow<'static, str>> {
 /// Installing is a simple matter of copying the running binary to
 /// `CARGO_HOME`/bin, hard-linking the various Rust tools to it,
 /// and adding `CARGO_HOME`/bin to PATH.
-pub fn install(
+pub(crate) fn install(
     no_prompt: bool,
     verbose: bool,
     quiet: bool,
@@ -352,12 +362,43 @@ pub fn install(
     let mut term = term2::stdout();
 
     #[cfg(windows)]
-    if !do_msvc_check(&opts) {
+    if let Some(plan) = do_msvc_check(&opts) {
         if no_prompt {
             warn!("installing msvc toolchain without its prerequisites");
+        } else if !quiet && plan == VsInstallPlan::Automatic {
+            md(&mut term, MSVC_AUTO_INSTALL_MESSAGE);
+            match windows::choose_vs_install()? {
+                Some(VsInstallPlan::Automatic) => {
+                    match try_install_msvc(&opts) {
+                        Err(e) => {
+                            // Make sure the console doesn't exit before the user can
+                            // see the error and give the option to continue anyway.
+                            report_error(&e);
+                            if !common::question_bool("\nContinue?", false)? {
+                                info!("aborting installation");
+                                return Ok(utils::ExitCode(0));
+                            }
+                        }
+                        Ok(ContinueInstall::No) => {
+                            ensure_prompt()?;
+                            return Ok(utils::ExitCode(0));
+                        }
+                        _ => {}
+                    }
+                }
+                Some(VsInstallPlan::Manual) => {
+                    md(&mut term, MSVC_MANUAL_INSTALL_MESSAGE);
+                    if !common::question_bool("\nContinue?", false)? {
+                        info!("aborting installation");
+                        return Ok(utils::ExitCode(0));
+                    }
+                }
+                None => {}
+            }
         } else {
             md(&mut term, MSVC_MESSAGE);
-            if !common::confirm("\nContinue? (y/N)", false)? {
+            md(&mut term, MSVC_MANUAL_INSTALL_MESSAGE);
+            if !common::question_bool("\nContinue?", false)? {
                 info!("aborting installation");
                 return Ok(utils::ExitCode(0));
             }
@@ -412,7 +453,7 @@ pub fn install(
     })();
 
     if let Err(e) = install_res {
-        common::report_error(&e);
+        report_error(&e);
 
         // On windows, where installation happens in a console
         // that may have opened just for this purpose, give
@@ -473,8 +514,8 @@ fn rustc_or_cargo_exists_in_path() -> Result<()> {
         let paths = env::split_paths(&paths).filter(ignore_paths);
 
         for path in paths {
-            let rustc = path.join(format!("rustc{}", EXE_SUFFIX));
-            let cargo = path.join(format!("cargo{}", EXE_SUFFIX));
+            let rustc = path.join(format!("rustc{EXE_SUFFIX}"));
+            let cargo = path.join(format!("cargo{EXE_SUFFIX}"));
 
             if rustc.exists() || cargo.exists() {
                 return Err(anyhow!("{}", path.to_str().unwrap().to_owned()));
@@ -496,7 +537,7 @@ fn check_existence_of_rustc_or_cargo_in_path(no_prompt: bool) -> Result<()> {
     if let Err(path) = rustc_or_cargo_exists_in_path() {
         warn!("it looks like you have an existing installation of Rust at:");
         warn!("{}", path);
-        warn!("rustup should not be installed alongside Rust. Please uninstall your existing Rust first.");
+        warn!("It is recommended that rustup be the primary Rust installation.");
         warn!("Otherwise you may have confusion unless you are careful with your PATH");
         warn!("If you are sure that you want both rustup and your already installed Rust");
         warn!("then please reply `y' or `yes' or set RUSTUP_INIT_SKIP_PATH_CHECK to yes");
@@ -549,7 +590,7 @@ fn do_pre_install_options_sanity_checks(opts: &InstallOpts<'_>) -> Result<()> {
         let toolchain_to_use = match &opts.default_toolchain {
             None => "stable",
             Some(s) if s == "none" => "stable",
-            Some(s) => &s,
+            Some(s) => s,
         };
         let partial_channel = dist::PartialToolchainDesc::from_str(toolchain_to_use)?;
         let resolved = partial_channel.resolve(&host_triple)?.to_string();
@@ -671,7 +712,7 @@ fn customize_install(mut opts: InstallOpts<'_>) -> Result<InstallOpts<'_>> {
 fn install_bins() -> Result<()> {
     let bin_path = utils::cargo_home()?.join("bin");
     let this_exe_path = utils::current_exe()?;
-    let rustup_path = bin_path.join(&format!("rustup{}", EXE_SUFFIX));
+    let rustup_path = bin_path.join(format!("rustup{EXE_SUFFIX}"));
 
     utils::ensure_dir_exists("bin", &bin_path, &|_: Notification<'_>| {})?;
     // NB: Even on Linux we can't just copy the new binary over the (running)
@@ -684,9 +725,9 @@ fn install_bins() -> Result<()> {
     install_proxies()
 }
 
-pub fn install_proxies() -> Result<()> {
+pub(crate) fn install_proxies() -> Result<()> {
     let bin_path = utils::cargo_home()?.join("bin");
-    let rustup_path = bin_path.join(&format!("rustup{}", EXE_SUFFIX));
+    let rustup_path = bin_path.join(&format!("rustup{EXE_SUFFIX}"));
 
     let rustup = Handle::from_path(&rustup_path)?;
 
@@ -715,7 +756,7 @@ pub fn install_proxies() -> Result<()> {
     // `tool_handles` later on. This'll allow us, afterwards, to actually
     // overwrite all the previous hard links with new ones.
     for tool in TOOLS {
-        let tool_path = bin_path.join(&format!("{}{}", tool, EXE_SUFFIX));
+        let tool_path = bin_path.join(&format!("{tool}{EXE_SUFFIX}"));
         if let Ok(handle) = Handle::from_path(&tool_path) {
             tool_handles.push(handle);
             if rustup == *tool_handles.last().unwrap() {
@@ -726,7 +767,7 @@ pub fn install_proxies() -> Result<()> {
     }
 
     for tool in DUP_TOOLS {
-        let tool_path = bin_path.join(&format!("{}{}", tool, EXE_SUFFIX));
+        let tool_path = bin_path.join(&format!("{tool}{EXE_SUFFIX}"));
         if let Ok(handle) = Handle::from_path(&tool_path) {
             // Like above, don't clobber anything that's already hardlinked to
             // avoid extraneous errors from being returned.
@@ -793,7 +834,7 @@ fn maybe_install_rust(
         let toolchain_str = toolchain.name().to_owned();
         toolchain.cfg().set_default(&toolchain_str)?;
         writeln!(process().stdout())?;
-        common::show_channel_update(&toolchain.cfg(), &toolchain_str, Ok(status))?;
+        common::show_channel_update(toolchain.cfg(), &toolchain_str, Ok(status))?;
     }
     Ok(())
 }
@@ -862,7 +903,7 @@ fn _install_selection<'a>(
     })
 }
 
-pub fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
+pub(crate) fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
     if NEVER_SELF_UPDATE {
         err!("self-uninstall is disabled for this build of rustup");
         err!("you should probably use your system package manager to uninstall rustup");
@@ -871,10 +912,7 @@ pub fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
 
     let cargo_home = utils::cargo_home()?;
 
-    if !cargo_home
-        .join(&format!("bin/rustup{}", EXE_SUFFIX))
-        .exists()
-    {
+    if !cargo_home.join(&format!("bin/rustup{EXE_SUFFIX}")).exists() {
         return Err(CLIError::NotSelfInstalled { p: cargo_home }.into());
     }
 
@@ -928,8 +966,8 @@ pub fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
     let tools = TOOLS
         .iter()
         .chain(DUP_TOOLS.iter())
-        .map(|t| format!("{}{}", t, EXE_SUFFIX));
-    let tools: Vec<_> = tools.chain(vec![format!("rustup{}", EXE_SUFFIX)]).collect();
+        .map(|t| format!("{t}{EXE_SUFFIX}"));
+    let tools: Vec<_> = tools.chain(vec![format!("rustup{EXE_SUFFIX}")]).collect();
     let bin_dir = cargo_home.join("bin");
     let diriter = fs::read_dir(&bin_dir).map_err(|e| CLIError::ReadDirError {
         p: bin_dir.clone(),
@@ -978,7 +1016,7 @@ pub fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
 /// (and on windows this process will not be running to do it),
 /// rustup-init is stored in `CARGO_HOME`/bin, and then deleted next
 /// time rustup runs.
-pub fn update(cfg: &Cfg) -> Result<utils::ExitCode> {
+pub(crate) fn update(cfg: &Cfg) -> Result<utils::ExitCode> {
     use common::SelfUpdatePermission::*;
     let update_permitted = if NEVER_SELF_UPDATE {
         HardFail
@@ -1048,10 +1086,10 @@ fn parse_new_rustup_version(version: String) -> String {
     String::from(matched_version)
 }
 
-pub fn prepare_update() -> Result<Option<PathBuf>> {
+pub(crate) fn prepare_update() -> Result<Option<PathBuf>> {
     let cargo_home = utils::cargo_home()?;
-    let rustup_path = cargo_home.join(&format!("bin{}rustup{}", MAIN_SEPARATOR, EXE_SUFFIX));
-    let setup_path = cargo_home.join(&format!("bin{}rustup-init{}", MAIN_SEPARATOR, EXE_SUFFIX));
+    let rustup_path = cargo_home.join(format!("bin{MAIN_SEPARATOR}rustup{EXE_SUFFIX}"));
+    let setup_path = cargo_home.join(format!("bin{MAIN_SEPARATOR}rustup-init{EXE_SUFFIX}"));
 
     if !rustup_path.exists() {
         return Err(CLIError::NotSelfInstalled { p: cargo_home }.into());
@@ -1082,7 +1120,7 @@ pub fn prepare_update() -> Result<Option<PathBuf>> {
     let current_version = env!("CARGO_PKG_VERSION");
 
     // Get available version
-    info!("checking for self-updates");
+    info!("checking for self-update");
     let available_version = get_available_rustup_version()?;
 
     // If up-to-date
@@ -1091,10 +1129,7 @@ pub fn prepare_update() -> Result<Option<PathBuf>> {
     }
 
     // Get download URL
-    let url = format!(
-        "{}/archive/{}/{}/rustup-init{}",
-        update_root, available_version, triple, EXE_SUFFIX
-    );
+    let url = format!("{update_root}/archive/{available_version}/{triple}/rustup-init{EXE_SUFFIX}");
 
     // Get download path
     let download_url = utils::parse_url(&url)?;
@@ -1109,7 +1144,7 @@ pub fn prepare_update() -> Result<Option<PathBuf>> {
     Ok(Some(setup_path))
 }
 
-pub fn get_available_rustup_version() -> Result<String> {
+pub(crate) fn get_available_rustup_version() -> Result<String> {
     let update_root = process()
         .var("RUSTUP_UPDATE_ROOT")
         .unwrap_or_else(|_| String::from(UPDATE_ROOT));
@@ -1119,7 +1154,7 @@ pub fn get_available_rustup_version() -> Result<String> {
         .context("error creating temp directory")?;
 
     // Parse the release file.
-    let release_file_url = format!("{}/release-stable.toml", update_root);
+    let release_file_url = format!("{update_root}/release-stable.toml");
     let release_file_url = utils::parse_url(&release_file_url)?;
     let release_file = tempdir.path().join("release-stable.toml");
     utils::download_file(&release_file_url, &release_file, None, &|_| ())?;
@@ -1135,8 +1170,7 @@ pub fn get_available_rustup_version() -> Result<String> {
         .ok_or_else(|| anyhow!("invalid schema key in rustup release file"))?;
     if schema != "1" {
         return Err(anyhow!(format!(
-            "unknown schema version '{}' in rustup release file",
-            schema
+            "unknown schema version '{schema}' in rustup release file"
         )));
     }
 
@@ -1150,7 +1184,7 @@ pub fn get_available_rustup_version() -> Result<String> {
     Ok(String::from(available_version))
 }
 
-pub fn check_rustup_update() -> Result<()> {
+pub(crate) fn check_rustup_update() -> Result<()> {
     let mut t = term2::stdout();
     // Get current rustup version
     let current_version = env!("CARGO_PKG_VERSION");
@@ -1165,20 +1199,20 @@ pub fn check_rustup_update() -> Result<()> {
         let _ = t.fg(term2::color::YELLOW);
         write!(t, "Update available")?;
         let _ = t.reset();
-        writeln!(t, " : {} -> {}", current_version, available_version)?;
+        writeln!(t, " : {current_version} -> {available_version}")?;
     } else {
         let _ = t.fg(term2::color::GREEN);
         write!(t, "Up to date")?;
         let _ = t.reset();
-        writeln!(t, " : {}", current_version)?;
+        writeln!(t, " : {current_version}")?;
     }
 
     Ok(())
 }
 
-pub fn cleanup_self_updater() -> Result<()> {
+pub(crate) fn cleanup_self_updater() -> Result<()> {
     let cargo_home = utils::cargo_home()?;
-    let setup = cargo_home.join(&format!("bin/rustup-init{}", EXE_SUFFIX));
+    let setup = cargo_home.join(format!("bin/rustup-init{EXE_SUFFIX}"));
 
     if setup.exists() {
         utils::remove_file("setup", &setup)?;
@@ -1187,10 +1221,10 @@ pub fn cleanup_self_updater() -> Result<()> {
     Ok(())
 }
 
-pub fn valid_self_update_modes() -> String {
+pub(crate) fn valid_self_update_modes() -> String {
     SelfUpdateMode::modes()
         .iter()
-        .map(|s| format!("'{}'", s))
+        .map(|s| format!("'{s}'"))
         .collect::<Vec<_>>()
         .join(", ")
 }
