@@ -1,5 +1,8 @@
-use std::env::consts::EXE_SUFFIX;
+use std::cell::RefCell;
+use std::env::{consts::EXE_SUFFIX, split_paths};
 use std::ffi::{OsStr, OsString};
+use std::fmt;
+use std::io::Write;
 use std::os::windows::ffi::{OsStrExt, OsStringExt};
 use std::path::Path;
 use std::process::Command;
@@ -9,6 +12,7 @@ use anyhow::{anyhow, Context, Result};
 use super::super::errors::*;
 use super::common;
 use super::{install_bins, InstallOpts};
+use crate::cli::download_tracker::DownloadTracker;
 use crate::dist::dist::TargetTriple;
 use crate::process;
 use crate::utils::utils;
@@ -17,19 +21,77 @@ use crate::utils::Notification;
 use winreg::enums::{RegType, HKEY_CURRENT_USER, KEY_READ, KEY_WRITE};
 use winreg::{RegKey, RegValue};
 
-pub fn ensure_prompt() -> Result<()> {
+pub(crate) fn ensure_prompt() -> Result<()> {
     writeln!(process().stdout(),)?;
     writeln!(process().stdout(), "Press the Enter key to continue.")?;
     common::read_line()?;
     Ok(())
 }
 
+fn choice(max: u8) -> Result<Option<u8>> {
+    write!(process().stdout(), ">")?;
+
+    let _ = std::io::stdout().flush();
+    let input = common::read_line()?;
+
+    let r = match str::parse(&input) {
+        Ok(n) if n <= max => Some(n),
+        _ => None,
+    };
+
+    writeln!(process().stdout())?;
+    Ok(r)
+}
+
+pub(crate) fn choose_vs_install() -> Result<Option<VsInstallPlan>> {
+    writeln!(
+        process().stdout(),
+        "\n1) Quick install via the Visual Studio Community installer"
+    )?;
+    writeln!(
+        process().stdout(),
+        "   (free for individuals, academic uses, and open source)."
+    )?;
+    writeln!(
+        process().stdout(),
+        "\n2) Manually install the prerequisites"
+    )?;
+    writeln!(
+        process().stdout(),
+        "   (for enterprise and advanced users)."
+    )?;
+    writeln!(process().stdout(), "\n3) Don't install the prerequisites")?;
+    writeln!(
+        process().stdout(),
+        "   (if you're targeting the GNU ABI).\n"
+    )?;
+
+    let choice = loop {
+        if let Some(n) = choice(3)? {
+            break n;
+        }
+        writeln!(process().stdout(), "Select option 1, 2 or 3")?;
+    };
+    let plan = match choice {
+        1 => Some(VsInstallPlan::Automatic),
+        2 => Some(VsInstallPlan::Manual),
+        _ => None,
+    };
+    Ok(plan)
+}
+
+#[derive(PartialEq, Eq)]
+pub(crate) enum VsInstallPlan {
+    Automatic,
+    Manual,
+}
+
 // Provide guidance about setting up MSVC if it doesn't appear to be
 // installed
-pub fn do_msvc_check(opts: &InstallOpts<'_>) -> bool {
+pub(crate) fn do_msvc_check(opts: &InstallOpts<'_>) -> Option<VsInstallPlan> {
     // Test suite skips this since it's env dependent
     if process().var("RUSTUP_INIT_SKIP_MSVC_CHECK").is_ok() {
-        return true;
+        return None;
     }
 
     use cc::windows_registry;
@@ -41,10 +103,159 @@ pub fn do_msvc_check(opts: &InstallOpts<'_>) -> bool {
     let installing_msvc = host_triple.contains("msvc");
     let have_msvc = windows_registry::find_tool(&host_triple, "cl.exe").is_some();
     if installing_msvc && !have_msvc {
-        return false;
+        // Visual Studio build tools are required.
+        // If the user does not have Visual Studio installed and their host
+        // machine is i686 or x86_64 then it's OK to try an auto install.
+        // Otherwise a manual install will be required.
+        let has_any_vs = windows_registry::find_vs_version().is_ok();
+        let is_x86 = host_triple.contains("i686") || host_triple.contains("x86_64");
+        if is_x86 && !has_any_vs {
+            Some(VsInstallPlan::Automatic)
+        } else {
+            Some(VsInstallPlan::Manual)
+        }
+    } else {
+        None
     }
+}
 
-    true
+#[derive(Debug, Eq, PartialEq)]
+struct VsInstallError(i32);
+impl std::error::Error for VsInstallError {}
+impl fmt::Display for VsInstallError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        // See https://docs.microsoft.com/en-us/visualstudio/install/use-command-line-parameters-to-install-visual-studio?view=vs-2022#error-codes
+        let message = match self.0 {
+            740 => "elevation required",
+            1001 => "Visual Studio installer process is running",
+            1003 => "Visual Studio is in use",
+            1602 => "operation was canceled",
+            1618 => "another installation running",
+            1641 => "operation completed successfully, and reboot was initiated",
+            3010 => "operation completed successfully, but install requires reboot before it can be used",
+            5003 => "bootstrapper failed to download installer",
+            5004 => "operation was canceled",
+            5005 => "bootstrapper command-line parse error",
+            5007 => "operation was blocked - the computer does not meet the requirements",
+            8001 => "arm machine check failure",
+            8002 => "background download precheck failure",
+            8003 => "out of support selectable failure",
+            8004 => "target directory failure",
+            8005 => "verifying source payloads failure",
+            8006 => "Visual Studio processes running",
+            -1073720687 => "connectivity failure",
+            -1073741510 => "Microsoft Visual Studio Installer was terminated",
+            _ => "error installing Visual Studio"
+        };
+        write!(f, "{} (exit code {})", message, self.0)
+    }
+}
+impl VsInstallError {
+    const REBOOTING_NOW: Self = Self(1641);
+    const REBOOT_REQUIRED: Self = Self(3010);
+}
+
+pub(crate) enum ContinueInstall {
+    Yes,
+    No,
+}
+
+/// Tries to install the needed Visual Studio components.
+///
+/// Returns `Ok(ContinueInstall::No)` if installing Visual Studio was successful
+/// but the rustup install should not be continued at this time.
+pub(crate) fn try_install_msvc(opts: &InstallOpts<'_>) -> Result<ContinueInstall> {
+    // download the installer
+    let visual_studio_url = utils::parse_url("https://aka.ms/vs/17/release/vs_community.exe")?;
+
+    let tempdir = tempfile::Builder::new()
+        .prefix("rustup-visualstudio")
+        .tempdir()
+        .context("error creating temp directory")?;
+
+    let visual_studio = tempdir.path().join("vs_setup.exe");
+    let download_tracker = RefCell::new(DownloadTracker::new().with_display_progress(true));
+    download_tracker.borrow_mut().download_finished();
+
+    info!("downloading Visual Studio installer");
+    utils::download_file(&visual_studio_url, &visual_studio, None, &move |n| {
+        download_tracker
+            .borrow_mut()
+            .handle_notification(&crate::Notification::Install(
+                crate::dist::Notification::Utils(n),
+            ));
+    })?;
+
+    // Run the installer. Arguments are documented at:
+    // https://docs.microsoft.com/en-us/visualstudio/install/use-command-line-parameters-to-install-visual-studio
+    let mut cmd = Command::new(visual_studio);
+    cmd.arg("--wait")
+        // Display an interactive GUI focused on installing just the selected components.
+        .arg("--focusedUi")
+        // Add the English language pack
+        .args(["--addProductLang", "En-us"])
+        // Add the linker and C runtime libraries.
+        .args(["--add", "Microsoft.VisualStudio.Component.VC.Tools.x86.x64"]);
+
+    // It's possible an earlier or later version of the Windows SDK has been
+    // installed separately from Visual Studio so installing it can be skipped.
+    if !has_windows_sdk_libs() {
+        cmd.args([
+            "--add",
+            "Microsoft.VisualStudio.Component.Windows11SDK.22000",
+        ]);
+    }
+    info!("running the Visual Studio install");
+    info!("rustup will continue once Visual Studio installation is complete\n");
+    let exit_status = cmd
+        .spawn()
+        .and_then(|mut child| child.wait())
+        .context("error running Visual Studio installer")?;
+
+    if exit_status.success() {
+        Ok(ContinueInstall::Yes)
+    } else {
+        match VsInstallError(exit_status.code().unwrap()) {
+            err @ VsInstallError::REBOOT_REQUIRED => {
+                // A reboot is required but the user opted to delay it.
+                warn!("{}", err);
+                Ok(ContinueInstall::Yes)
+            }
+            err @ VsInstallError::REBOOTING_NOW => {
+                // The user is wanting to reboot right now, so we should
+                // not continue the install.
+                warn!("{}", err);
+                info!("\nRun rustup-init after restart to continue install");
+                Ok(ContinueInstall::No)
+            }
+            err => {
+                // It's possible that the installer returned a non-zero exit code
+                // even though the required components were successfully installed.
+                // In that case we warn about the error but continue on.
+                let have_msvc = do_msvc_check(opts).is_none();
+                let has_libs = has_windows_sdk_libs();
+                if have_msvc && has_libs {
+                    warn!("Visual Studio is installed but a problem occurred during installation");
+                    warn!("{}", err);
+                    Ok(ContinueInstall::Yes)
+                } else {
+                    Err(err).context("failed to install Visual Studio")
+                }
+            }
+        }
+    }
+}
+
+fn has_windows_sdk_libs() -> bool {
+    if let Some(paths) = process().var_os("lib") {
+        for mut path in split_paths(&paths) {
+            path.push("kernel32.lib");
+            if path.exists() {
+                return true;
+            }
+        }
+    };
+    false
 }
 
 /// Run by rustup-gc-$num.exe to delete CARGO_HOME
@@ -72,7 +283,7 @@ pub fn complete_windows_uninstall() -> Result<utils::ExitCode> {
     Ok(utils::ExitCode(0))
 }
 
-pub fn wait_for_parent() -> Result<()> {
+pub(crate) fn wait_for_parent() -> Result<()> {
     use std::io;
     use std::mem;
     use winapi::shared::minwindef::DWORD;
@@ -145,7 +356,7 @@ pub fn wait_for_parent() -> Result<()> {
     Ok(())
 }
 
-pub fn do_add_to_path() -> Result<()> {
+pub(crate) fn do_add_to_path() -> Result<()> {
     let new_path = _with_path_cargo_home_bin(_add_to_path)?;
     _apply_new_path(new_path)
 }
@@ -246,7 +457,7 @@ fn _remove_from_path(old_path: Vec<u16>, path_str: Vec<u16>) -> Option<Vec<u16>>
         .position(|path| path == path_str)?;
     // If there's a trailing semicolon (likely, since we probably added one
     // during install), include that in the substring to remove. We don't search
-    // for that to find the string, because if its the last string in the path,
+    // for that to find the string, because if it's the last string in the path,
     // there may not be.
     let mut len = path_str.len();
     if old_path.get(idx + path_str.len()) == Some(&(b';' as u16)) {
@@ -274,14 +485,14 @@ where
         .and_then(|old_path| f(old_path, OsString::from(path_str).encode_wide().collect())))
 }
 
-pub fn do_remove_from_path() -> Result<()> {
+pub(crate) fn do_remove_from_path() -> Result<()> {
     let new_path = _with_path_cargo_home_bin(_remove_from_path)?;
     _apply_new_path(new_path)
 }
 
 const RUSTUP_UNINSTALL_ENTRY: &str = r"Software\Microsoft\Windows\CurrentVersion\Uninstall\Rustup";
 
-pub fn do_add_to_programs() -> Result<()> {
+pub(crate) fn do_add_to_programs() -> Result<()> {
     use std::path::PathBuf;
 
     let key = RegKey::predef(HKEY_CURRENT_USER)
@@ -312,15 +523,19 @@ pub fn do_add_to_programs() -> Result<()> {
         vtype: RegType::REG_SZ,
     };
 
+    let current_version: &str = env!("CARGO_PKG_VERSION");
+
     key.set_raw_value("UninstallString", &reg_value)
         .context("Failed to set uninstall string")?;
     key.set_value("DisplayName", &"Rustup: the Rust toolchain installer")
         .context("Failed to set display name")?;
+    key.set_value("DisplayVersion", &current_version)
+        .context("Failed to set display version")?;
 
     Ok(())
 }
 
-pub fn do_remove_from_programs() -> Result<()> {
+pub(crate) fn do_remove_from_programs() -> Result<()> {
     match RegKey::predef(HKEY_CURRENT_USER).delete_subkey_all(RUSTUP_UNINSTALL_ENTRY) {
         Ok(()) => Ok(()),
         Err(ref e) if e.kind() == std::io::ErrorKind::NotFound => Ok(()),
@@ -329,7 +544,7 @@ pub fn do_remove_from_programs() -> Result<()> {
 }
 
 /// Convert a vector UCS-2 chars to a null-terminated UCS-2 string in bytes
-pub fn to_winreg_bytes(mut v: Vec<u16>) -> Vec<u8> {
+pub(crate) fn to_winreg_bytes(mut v: Vec<u16>) -> Vec<u8> {
     v.push(0);
     unsafe { std::slice::from_raw_parts(v.as_ptr().cast::<u8>(), v.len() * 2).to_vec() }
 }
@@ -337,7 +552,7 @@ pub fn to_winreg_bytes(mut v: Vec<u16>) -> Vec<u8> {
 /// This is used to decode the value of HKCU\Environment\PATH. If that key is
 /// not REG_SZ | REG_EXPAND_SZ then this returns None. The winreg library itself
 /// does a lossy unicode conversion.
-pub fn from_winreg_value(val: &winreg::RegValue) -> Option<Vec<u16>> {
+pub(crate) fn from_winreg_value(val: &winreg::RegValue) -> Option<Vec<u16>> {
     use std::slice;
 
     match val.vtype {
@@ -357,7 +572,7 @@ pub fn from_winreg_value(val: &winreg::RegValue) -> Option<Vec<u16>> {
     }
 }
 
-pub fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
+pub(crate) fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
     Command::new(setup_path)
         .arg("--self-replace")
         .spawn()
@@ -366,7 +581,7 @@ pub fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
     Ok(utils::ExitCode(0))
 }
 
-pub fn self_replace() -> Result<utils::ExitCode> {
+pub(crate) fn self_replace() -> Result<utils::ExitCode> {
     wait_for_parent()?;
     install_bins()?;
 
@@ -403,7 +618,7 @@ pub fn self_replace() -> Result<utils::ExitCode> {
 //
 // .. augmented with this SO answer
 // https://stackoverflow.com/questions/10319526/understanding-a-self-deleting-program-in-c
-pub fn delete_rustup_and_cargo_home() -> Result<()> {
+pub(crate) fn delete_rustup_and_cargo_home() -> Result<()> {
     use std::io;
     use std::mem;
     use std::ptr;
@@ -419,7 +634,7 @@ pub fn delete_rustup_and_cargo_home() -> Result<()> {
     // CARGO_HOME, hopefully empty except for bin/rustup.exe
     let cargo_home = utils::cargo_home()?;
     // The rustup.exe bin
-    let rustup_path = cargo_home.join(&format!("bin/rustup{}", EXE_SUFFIX));
+    let rustup_path = cargo_home.join(format!("bin/rustup{EXE_SUFFIX}"));
 
     // The directory containing CARGO_HOME
     let work_path = cargo_home
@@ -429,7 +644,7 @@ pub fn delete_rustup_and_cargo_home() -> Result<()> {
     // Generate a unique name for the files we're about to move out
     // of CARGO_HOME.
     let numbah: u32 = rand::random();
-    let gc_exe = work_path.join(&format!("rustup-gc-{:x}.exe", numbah));
+    let gc_exe = work_path.join(format!("rustup-gc-{numbah:x}.exe"));
     // Copy rustup (probably this process's exe) to the gc exe
     utils::copy_file(&rustup_path, &gc_exe)?;
     let gc_exe_win: Vec<_> = gc_exe.as_os_str().encode_wide().chain(Some(0)).collect();
@@ -530,7 +745,7 @@ mod tests {
     fn windows_path_regkey_type() {
         // per issue #261, setting PATH should use REG_EXPAND_SZ.
         let tp = Box::new(currentprocess::TestProcess::default());
-        with_saved_path(&|| {
+        with_saved_path(&mut || {
             currentprocess::with(tp.clone(), || {
                 let root = RegKey::predef(HKEY_CURRENT_USER);
                 let environment = root
@@ -560,7 +775,7 @@ mod tests {
         // during uninstall the PATH key may end up empty; if so we should
         // delete it.
         let tp = Box::new(currentprocess::TestProcess::default());
-        with_saved_path(&|| {
+        with_saved_path(&mut || {
             currentprocess::with(tp.clone(), || {
                 let root = RegKey::predef(HKEY_CURRENT_USER);
                 let environment = root
@@ -585,7 +800,7 @@ mod tests {
                 match reg_value {
                     Ok(_) => panic!("key not deleted"),
                     Err(ref e) if e.kind() == io::ErrorKind::NotFound => {}
-                    Err(ref e) => panic!("error {}", e),
+                    Err(ref e) => panic!("error {e}"),
                 }
             })
         });
@@ -601,7 +816,7 @@ mod tests {
                 .collect(),
             ..Default::default()
         });
-        with_saved_path(&|| {
+        with_saved_path(&mut || {
             currentprocess::with(tp.clone(), || {
                 let root = RegKey::predef(HKEY_CURRENT_USER);
                 let environment = root
@@ -630,7 +845,7 @@ mod tests {
     fn windows_treat_missing_path_as_empty() {
         // during install the PATH key may be missing; treat it as empty
         let tp = Box::new(currentprocess::TestProcess::default());
-        with_saved_path(&|| {
+        with_saved_path(&mut || {
             currentprocess::with(tp.clone(), || {
                 let root = RegKey::predef(HKEY_CURRENT_USER);
                 let environment = root
