@@ -7,14 +7,13 @@ use std::str::FromStr;
 use std::sync::Arc;
 
 use anyhow::{anyhow, bail, Context, Result};
-use pgp::{Deserializable, SignedPublicKey};
 use serde::Deserialize;
 use thiserror::Error as ThisError;
 
 use crate::cli::self_update::SelfUpdateMode;
 use crate::dist::download::DownloadCfg;
 use crate::dist::{
-    dist::{self, valid_profile_names},
+    dist::{self, Profile},
     temp,
 };
 use crate::errors::RustupError;
@@ -86,7 +85,7 @@ impl<T: Into<String>> From<T> for OverrideFile {
 }
 
 #[derive(Debug)]
-pub enum OverrideReason {
+pub(crate) enum OverrideReason {
     Environment,
     CommandLine,
     OverrideDB(PathBuf),
@@ -154,82 +153,10 @@ impl<'a> OverrideCfg<'a> {
     }
 }
 
-lazy_static::lazy_static! {
-    static ref BUILTIN_PGP_KEY: SignedPublicKey = pgp::SignedPublicKey::from_armor_single(
-        io::Cursor::new(&include_bytes!("rust-key.pgp.ascii")[..])
-    ).unwrap().0;
-}
-
-#[allow(clippy::large_enum_variant)] // Builtin is tiny, the rest are sane
-#[derive(Debug)]
-pub enum PgpPublicKey {
-    Builtin,
-    FromEnvironment(PathBuf, SignedPublicKey),
-    FromConfiguration(PathBuf, SignedPublicKey),
-}
-
-impl PgpPublicKey {
-    /// Retrieve the key.
-    pub fn key(&self) -> &SignedPublicKey {
-        match self {
-            Self::Builtin => &*BUILTIN_PGP_KEY,
-            Self::FromEnvironment(_, k) => &k,
-            Self::FromConfiguration(_, k) => &k,
-        }
-    }
-
-    /// Display the key in detail for the user
-    pub fn show_key(&self) -> Result<Vec<String>> {
-        fn format_hex(bytes: &[u8], separator: &str, every: usize) -> Result<String> {
-            use std::fmt::Write;
-            let mut ret = String::new();
-            let mut wait = every;
-            for b in bytes.iter() {
-                if wait == 0 {
-                    ret.push_str(separator);
-                    wait = every;
-                }
-                wait -= 1;
-                write!(ret, "{:02X}", b)?;
-            }
-            Ok(ret)
-        }
-        use pgp::types::KeyTrait;
-        let mut ret = vec![format!("from {}", self)];
-        let key = self.key();
-        let keyid = format_hex(&key.key_id().to_vec(), "-", 4)?;
-        let algo = key.algorithm();
-        let fpr = format_hex(&key.fingerprint(), " ", 2)?;
-        let uid0 = key
-            .details
-            .users
-            .get(0)
-            .map(|u| u.id.id())
-            .unwrap_or("<No User ID>");
-        ret.push(format!("  {:?}/{} - {}", algo, keyid, uid0));
-        ret.push(format!("  Fingerprint: {}", fpr));
-        Ok(ret)
-    }
-}
-
-impl Display for PgpPublicKey {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            Self::Builtin => write!(f, "builtin Rust release key"),
-            Self::FromEnvironment(p, _) => {
-                write!(f, "key specified in RUSTUP_PGP_KEY ({})", p.display())
-            }
-            Self::FromConfiguration(p, _) => {
-                write!(f, "key specified in configuration file ({})", p.display())
-            }
-        }
-    }
-}
-
-pub const UNIX_FALLBACK_SETTINGS: &str = "/etc/rustup/settings.toml";
+pub(crate) const UNIX_FALLBACK_SETTINGS: &str = "/etc/rustup/settings.toml";
 
 pub struct Cfg {
-    pub profile_override: Option<dist::Profile>,
+    profile_override: Option<dist::Profile>,
     pub rustup_dir: PathBuf,
     pub settings_file: SettingsFile,
     pub fallback_settings: Option<FallbackSettings>,
@@ -237,16 +164,14 @@ pub struct Cfg {
     pub update_hash_dir: PathBuf,
     pub download_dir: PathBuf,
     pub temp_cfg: temp::Cfg,
-    pgp_keys: Vec<PgpPublicKey>,
     pub toolchain_override: Option<String>,
     pub env_override: Option<String>,
     pub dist_root_url: String,
-    pub dist_root_server: String,
     pub notify_handler: Arc<dyn Fn(Notification<'_>)>,
 }
 
 impl Cfg {
-    pub fn from_env(notify_handler: Arc<dyn Fn(Notification<'_>)>) -> Result<Self> {
+    pub(crate) fn from_env(notify_handler: Arc<dyn Fn(Notification<'_>)>) -> Result<Self> {
         // Set up the rustup home directory
         let rustup_dir = utils::rustup_home()?;
 
@@ -272,37 +197,6 @@ impl Cfg {
         let update_hash_dir = rustup_dir.join("update-hashes");
         let download_dir = rustup_dir.join("downloads");
 
-        // PGP keys
-        let mut pgp_keys: Vec<PgpPublicKey> = vec![PgpPublicKey::Builtin];
-
-        if let Some(ref s_path) = process().var_os("RUSTUP_PGP_KEY") {
-            let path = PathBuf::from(s_path);
-            let file = utils::open_file("RUSTUP_PGP_KEY", &path)?;
-            let (key, _) = SignedPublicKey::from_armor_single(file).map_err(|error| {
-                RustupError::InvalidPgpKey {
-                    path: s_path.into(),
-                    source: error,
-                }
-            })?;
-
-            pgp_keys.push(PgpPublicKey::FromEnvironment(path, key));
-        }
-        settings_file.with(|s| {
-            if let Some(s) = &s.pgp_keys {
-                let path = PathBuf::from(s);
-                let file = utils::open_file("PGP Key from config", &path)?;
-                let (key, _) = SignedPublicKey::from_armor_single(file).map_err(|error| {
-                    anyhow!(RustupError::InvalidPgpKey {
-                        path: s.into(),
-                        source: error,
-                    })
-                })?;
-
-                pgp_keys.push(PgpPublicKey::FromConfiguration(path, key));
-            }
-            Ok(())
-        })?;
-
         // Environment override
         let env_override = process()
             .var("RUSTUP_TOOLCHAIN")
@@ -330,7 +224,7 @@ impl Cfg {
             dist_root_server.as_str(),
             Box::new(move |n| (notify_clone)(n.into())),
         );
-        let dist_root = dist_root_server.clone() + "/dist";
+        let dist_root = dist_root_server + "/dist";
 
         let cfg = Self {
             profile_override: None,
@@ -341,12 +235,10 @@ impl Cfg {
             update_hash_dir,
             download_dir,
             temp_cfg,
-            pgp_keys,
             notify_handler,
             toolchain_override: None,
             env_override,
             dist_root_url: dist_root,
-            dist_root_server,
         };
 
         // Run some basic checks against the constructed configuration
@@ -359,7 +251,7 @@ impl Cfg {
     }
 
     /// construct a download configuration
-    pub fn download_cfg<'a>(
+    pub(crate) fn download_cfg<'a>(
         &'a self,
         notify_handler: &'a dyn Fn(crate::dist::Notification<'_>),
     ) -> DownloadCfg<'a> {
@@ -368,45 +260,42 @@ impl Cfg {
             temp_cfg: &self.temp_cfg,
             download_dir: &self.download_dir,
             notify_handler,
-            pgp_keys: self.get_pgp_keys(),
         }
     }
 
-    pub fn get_pgp_keys(&self) -> &[PgpPublicKey] {
-        &self.pgp_keys
-    }
-
-    pub fn set_profile_override(&mut self, profile: dist::Profile) {
+    pub(crate) fn set_profile_override(&mut self, profile: dist::Profile) {
         self.profile_override = Some(profile);
     }
 
-    pub fn set_default(&self, toolchain: &str) -> Result<()> {
+    pub(crate) fn set_default(&self, toolchain: &str) -> Result<()> {
         self.settings_file.with_mut(|s| {
-            s.default_toolchain = Some(toolchain.to_owned());
+            s.default_toolchain = if toolchain == "none" {
+                None
+            } else {
+                Some(toolchain.to_owned())
+            };
             Ok(())
         })?;
         (self.notify_handler)(Notification::SetDefaultToolchain(toolchain));
         Ok(())
     }
 
-    pub fn set_profile(&mut self, profile: &str) -> Result<()> {
-        if !dist::Profile::names().contains(&profile) {
-            return Err(anyhow!(
-                "unknown profile name: '{}'; valid profile names are {}",
-                profile.to_owned(),
-                valid_profile_names(),
-            ));
+    pub(crate) fn set_profile(&mut self, profile: &str) -> Result<()> {
+        match Profile::from_str(profile) {
+            Ok(p) => {
+                self.profile_override = None;
+                self.settings_file.with_mut(|s| {
+                    s.profile = Some(p);
+                    Ok(())
+                })?;
+                (self.notify_handler)(Notification::SetProfile(profile));
+                Ok(())
+            }
+            Err(err) => Err(err),
         }
-        self.profile_override = None;
-        self.settings_file.with_mut(|s| {
-            s.profile = Some(profile.to_owned());
-            Ok(())
-        })?;
-        (self.notify_handler)(Notification::SetProfile(profile));
-        Ok(())
     }
 
-    pub fn set_auto_self_update(&mut self, mode: &str) -> Result<()> {
+    pub(crate) fn set_auto_self_update(&mut self, mode: &str) -> Result<()> {
         match SelfUpdateMode::from_str(mode) {
             Ok(update_mode) => {
                 self.settings_file.with_mut(|s| {
@@ -420,32 +309,31 @@ impl Cfg {
         }
     }
 
-    pub fn set_toolchain_override(&mut self, toolchain_override: &str) {
+    pub(crate) fn set_toolchain_override(&mut self, toolchain_override: &str) {
         self.toolchain_override = Some(toolchain_override.to_owned());
     }
 
     // Returns a profile, if one exists in the settings file.
     //
     // Returns `Err` if the settings file could not be read or the profile is
-    // invalid. Returns `Ok(...)` if there is a valid profile, and `Ok(Profile::default())`
+    // invalid. Returns `Ok(...)` if there is a valid profile, and `Ok(Profile::Default)`
     // if there is no profile in the settings file. The last variant happens when
     // a user upgrades from a version of Rustup without profiles to a version of
     // Rustup with profiles.
-    pub fn get_profile(&self) -> Result<dist::Profile> {
+    pub(crate) fn get_profile(&self) -> Result<dist::Profile> {
         if let Some(p) = self.profile_override {
             return Ok(p);
         }
         self.settings_file.with(|s| {
-            let p = match &s.profile {
+            let p = match s.profile {
                 Some(p) => p,
-                None => dist::Profile::default_name(),
+                None => Profile::Default,
             };
-            let p = dist::Profile::from_str(p)?;
             Ok(p)
         })
     }
 
-    pub fn get_self_update_mode(&self) -> Result<SelfUpdateMode> {
+    pub(crate) fn get_self_update_mode(&self) -> Result<SelfUpdateMode> {
         self.settings_file.with(|s| {
             let mode = match &s.auto_self_update {
                 Some(mode) => mode.clone(),
@@ -455,17 +343,21 @@ impl Cfg {
         })
     }
 
-    pub fn get_toolchain(&self, name: &str, create_parent: bool) -> Result<Toolchain<'_>> {
+    pub(crate) fn get_toolchain(&self, name: &str, create_parent: bool) -> Result<Toolchain<'_>> {
         if create_parent {
             utils::ensure_dir_exists("toolchains", &self.toolchains_dir, &|n| {
                 (self.notify_handler)(n)
             })?;
         }
 
+        if name.is_empty() {
+            anyhow::bail!("toolchain names must not be empty");
+        }
+
         Toolchain::from(self, name)
     }
 
-    pub fn get_hash_file(&self, toolchain: &str, create_parent: bool) -> Result<PathBuf> {
+    pub(crate) fn get_hash_file(&self, toolchain: &str, create_parent: bool) -> Result<PathBuf> {
         if create_parent {
             utils::ensure_dir_exists(
                 "update-hash",
@@ -477,25 +369,25 @@ impl Cfg {
         Ok(self.update_hash_dir.join(toolchain))
     }
 
-    pub fn which_binary_by_toolchain(
+    pub(crate) fn which_binary_by_toolchain(
         &self,
-        toolchain: &str,
+        toolchain_name: &str,
         binary: &str,
-    ) -> Result<Option<PathBuf>> {
-        let toolchain = self.get_toolchain(toolchain, false)?;
+    ) -> Result<PathBuf> {
+        let toolchain = self.get_toolchain(toolchain_name, false)?;
         if toolchain.exists() {
-            Ok(Some(toolchain.binary_file(binary)))
+            Ok(toolchain.binary_file(binary))
         } else {
-            Ok(None)
+            Err(RustupError::ToolchainNotInstalled(toolchain_name.to_string()).into())
         }
     }
 
-    pub fn which_binary(&self, path: &Path, binary: &str) -> Result<Option<PathBuf>> {
+    pub(crate) fn which_binary(&self, path: &Path, binary: &str) -> Result<PathBuf> {
         let (toolchain, _) = self.find_or_install_override_toolchain_or_default(path)?;
-        Ok(Some(toolchain.binary_file(binary)))
+        Ok(toolchain.binary_file(binary))
     }
 
-    pub fn upgrade_data(&self) -> Result<()> {
+    pub(crate) fn upgrade_data(&self) -> Result<()> {
         let current_version = self.settings_file.with(|s| Ok(s.version.clone()))?;
 
         if current_version == DEFAULT_METADATA_VERSION {
@@ -535,7 +427,7 @@ impl Cfg {
         }
     }
 
-    pub fn find_default(&self) -> Result<Option<Toolchain<'_>>> {
+    pub(crate) fn find_default(&self) -> Result<Option<Toolchain<'_>>> {
         let opt_name = self.get_default()?;
 
         if let Some(name) = opt_name {
@@ -546,7 +438,10 @@ impl Cfg {
         }
     }
 
-    pub fn find_override(&self, path: &Path) -> Result<Option<(Toolchain<'_>, OverrideReason)>> {
+    pub(crate) fn find_override(
+        &self,
+        path: &Path,
+    ) -> Result<Option<(Toolchain<'_>, OverrideReason)>> {
         self.find_override_config(path).map(|opt| {
             opt.and_then(|(override_cfg, reason)| {
                 override_cfg.toolchain.map(|toolchain| (toolchain, reason))
@@ -675,13 +570,16 @@ impl Cfg {
             };
 
             if let Ok(contents) = contents {
-                let override_file = Cfg::parse_override_file(contents, parse_mode)?;
+                let add_file_context = || format!("in {}", toolchain_file.to_string_lossy());
+                let override_file = Cfg::parse_override_file(contents, parse_mode)
+                    .with_context(add_file_context)?;
                 if let Some(toolchain_name) = &override_file.toolchain.channel {
-                    let all_toolchains = self.list_toolchains()?;
+                    let all_toolchains = self.list_toolchains().with_context(add_file_context)?;
                     if !all_toolchains.iter().any(|s| s == toolchain_name) {
                         // The given name is not resolvable as a toolchain, so
                         // instead check it's plausible for installation later
-                        dist::validate_channel_name(&toolchain_name)?;
+                        dist::validate_channel_name(toolchain_name)
+                            .with_context(add_file_context)?;
                     }
                 }
 
@@ -725,7 +623,7 @@ impl Cfg {
         }
     }
 
-    pub fn find_or_install_override_toolchain_or_default(
+    pub(crate) fn find_or_install_override_toolchain_or_default(
         &self,
         path: &Path,
     ) -> Result<(Toolchain<'_>, Option<OverrideReason>)> {
@@ -737,21 +635,21 @@ impl Cfg {
             let components_requested = !components.is_empty() || !targets.is_empty();
             // If we're here, the toolchain exists on disk and is a dist toolchain
             // so we should attempt to load its manifest
-            let manifest = if let Some(manifest) = distributable.get_manifest()? {
-                manifest
+            let desc = if let Some(desc) = distributable.get_toolchain_desc_with_manifest()? {
+                desc
             } else {
                 // We can't read the manifest.  If this is a v1 install that's understandable
                 // and we assume the components are all good, otherwise we need to have a go
                 // at re-fetching the manifest to try again.
                 return Ok(distributable.guess_v1_manifest());
             };
-            match (distributable.list_components(), components_requested) {
+            match (desc.list_components(), components_requested) {
                 // If the toolchain does not support components but there were components requested, bubble up the error
                 (Err(e), true) => Err(e),
                 // Otherwise check if all the components we want are installed
                 (Ok(installed_components), _) => Ok(components.iter().all(|name| {
                     installed_components.iter().any(|status| {
-                        let cname = status.component.short_name(&manifest);
+                        let cname = status.component.short_name(&desc.manifest);
                         let cname = cname.as_str();
                         let cnameim = status.component.short_name_in_manifest();
                         let cnameim = cnameim.as_str();
@@ -822,7 +720,7 @@ impl Cfg {
         }
     }
 
-    pub fn get_default(&self) -> Result<Option<String>> {
+    pub(crate) fn get_default(&self) -> Result<Option<String>> {
         let user_opt = self.settings_file.with(|s| Ok(s.default_toolchain.clone()));
         if let Some(fallback_settings) = &self.fallback_settings {
             match user_opt {
@@ -833,7 +731,7 @@ impl Cfg {
         user_opt
     }
 
-    pub fn list_toolchains(&self) -> Result<Vec<String>> {
+    pub(crate) fn list_toolchains(&self) -> Result<Vec<String>> {
         if utils::is_directory(&self.toolchains_dir) {
             let mut toolchains: Vec<_> = utils::read_dir("toolchains", &self.toolchains_dir)?
                 .filter_map(io::Result::ok)
@@ -849,7 +747,7 @@ impl Cfg {
         }
     }
 
-    pub fn list_channels(&self) -> Result<Vec<(String, Result<Toolchain<'_>>)>> {
+    pub(crate) fn list_channels(&self) -> Result<Vec<(String, Result<Toolchain<'_>>)>> {
         let toolchains = self.list_toolchains()?;
 
         // Convert the toolchain strings to Toolchain values
@@ -858,11 +756,11 @@ impl Cfg {
 
         // Filter out toolchains that don't track a release channel
         Ok(toolchains
-            .filter(|&(_, ref t)| t.as_ref().map(Toolchain::is_tracking).unwrap_or(false))
+            .filter(|(_, ref t)| t.as_ref().map(Toolchain::is_tracking).unwrap_or(false))
             .collect())
     }
 
-    pub fn update_all_channels(
+    pub(crate) fn update_all_channels(
         &self,
         force_update: bool,
     ) -> Result<Vec<(String, Result<UpdateStatus>)>> {
@@ -886,7 +784,7 @@ impl Cfg {
         Ok(channels.collect())
     }
 
-    pub fn check_metadata_version(&self) -> Result<()> {
+    pub(crate) fn check_metadata_version(&self) -> Result<()> {
         utils::assert_is_directory(&self.rustup_dir)?;
 
         self.settings_file.with(|s| {
@@ -901,14 +799,14 @@ impl Cfg {
         })
     }
 
-    pub fn toolchain_for_dir(
+    pub(crate) fn toolchain_for_dir(
         &self,
         path: &Path,
     ) -> Result<(Toolchain<'_>, Option<OverrideReason>)> {
         self.find_or_install_override_toolchain_or_default(path)
     }
 
-    pub fn create_command_for_dir(&self, path: &Path, binary: &str) -> Result<Command> {
+    pub(crate) fn create_command_for_dir(&self, path: &Path, binary: &str) -> Result<Command> {
         let (ref toolchain, _) = self.toolchain_for_dir(path)?;
 
         if let Some(cmd) = self.maybe_do_cargo_fallback(toolchain, binary)? {
@@ -921,7 +819,7 @@ impl Cfg {
         }
     }
 
-    pub fn create_command_for_toolchain(
+    pub(crate) fn create_command_for_toolchain(
         &self,
         toolchain: &str,
         install_if_missing: bool,
@@ -977,7 +875,7 @@ impl Cfg {
         Ok(None)
     }
 
-    pub fn set_default_host_triple(&self, host_triple: &str) -> Result<()> {
+    pub(crate) fn set_default_host_triple(&self, host_triple: &str) -> Result<()> {
         // Ensure that the provided host_triple is capable of resolving
         // against the 'stable' toolchain.  This provides early errors
         // if the supplied triple is insufficient / bad.
@@ -989,23 +887,25 @@ impl Cfg {
         })
     }
 
-    pub fn get_default_host_triple(&self) -> Result<dist::TargetTriple> {
+    pub(crate) fn get_default_host_triple(&self) -> Result<dist::TargetTriple> {
         Ok(self
             .settings_file
             .with(|s| {
                 Ok(s.default_host_triple
                     .as_ref()
-                    .map(|s| dist::TargetTriple::new(&s)))
+                    .map(|s| dist::TargetTriple::new(s)))
             })?
             .unwrap_or_else(dist::TargetTriple::from_host_or_build))
     }
 
-    pub fn resolve_toolchain(&self, name: &str) -> Result<String> {
-        if let Ok(desc) = dist::PartialToolchainDesc::from_str(name) {
+    pub(crate) fn resolve_toolchain(&self, name: &str) -> Result<String> {
+        // remove trailing slashes in toolchain name
+        let normalized_name = name.trim_end_matches('/');
+        if let Ok(desc) = dist::PartialToolchainDesc::from_str(normalized_name) {
             let host = self.get_default_host_triple()?;
             Ok(desc.resolve(&host)?.to_string())
         } else {
-            Ok(name.to_owned())
+            Ok(normalized_name.to_owned())
         }
     }
 }
