@@ -108,7 +108,7 @@ impl Deref for FileBuffer {
 
     fn deref(&self) -> &Self::Target {
         match self {
-            FileBuffer::Immediate(ref vec) => &vec,
+            FileBuffer::Immediate(ref vec) => vec,
             FileBuffer::Threaded(PoolReference::Owned(owned, _)) => owned,
             FileBuffer::Threaded(PoolReference::Mut(mutable, _)) => mutable,
         }
@@ -193,8 +193,6 @@ pub(crate) struct Item {
     start: Option<Instant>,
     /// Amount of time the operation took to finish
     finish: Option<Duration>,
-    /// The length of the file, for files (for stats)
-    size: Option<usize>,
     /// The result of the operation (could now be factored into CompletedIO...)
     pub(crate) result: io::Result<()>,
     /// The mode to apply
@@ -216,20 +214,17 @@ impl Item {
             kind: Kind::Directory,
             start: None,
             finish: None,
-            size: None,
             result: Ok(()),
             mode,
         }
     }
 
     pub(crate) fn write_file(full_path: PathBuf, mode: u32, content: FileBuffer) -> Self {
-        let len = content.len();
         Self {
             full_path,
             kind: Kind::File(content),
             start: None,
             finish: None,
-            size: Some(len),
             result: Ok(()),
             mode,
         }
@@ -246,7 +241,6 @@ impl Item {
             kind: Kind::IncrementalFile(content_callback),
             start: None,
             finish: None,
-            size: None,
             result: Ok(()),
             mode,
         };
@@ -304,7 +298,7 @@ pub(crate) trait Executor {
         self.dispatch(item)
     }
 
-    /// Actually dispatch a operation.
+    /// Actually dispatch an operation.
     /// This is called by the default execute() implementation and
     /// should not be called directly.
     fn dispatch(&self, item: Item) -> Box<dyn Iterator<Item = CompletedIo> + '_>;
@@ -345,10 +339,10 @@ pub(crate) fn perform<F: Fn(usize)>(item: &mut Item, chunk_complete_callback: F)
             contents.clear();
             match contents {
                 FileBuffer::Immediate(ref contents) => {
-                    write_file(&item.full_path, &contents, item.mode)
+                    write_file(&item.full_path, contents, item.mode)
                 }
                 FileBuffer::Threaded(ref mut contents) => {
-                    write_file(&item.full_path, &contents, item.mode)
+                    write_file(&item.full_path, contents, item.mode)
                 }
             }
         }
@@ -417,7 +411,7 @@ pub(crate) fn write_file_incremental<P: AsRef<Path>, F: Fn(usize)>(
     if let IncrementalFile::ThreadedReceiver(recv) = content_callback {
         loop {
             // We unwrap here because the documented only reason for recv to fail is a close by the sender, which is reading
-            // from the tar file: a failed read there will propogate the error in the main thread directly.
+            // from the tar file: a failed read there will propagate the error in the main thread directly.
             let contents = recv.recv().unwrap();
             let len = contents.len();
             // Length 0 vector is used for clean EOF signalling.
