@@ -6,21 +6,13 @@ use anyhow::{Context, Result};
 
 use crate::utils::utils;
 
-#[derive(Clone, Debug, Deserialize, PartialEq)]
+#[derive(Clone, Debug, Deserialize, Eq, PartialEq, Default)]
 pub struct FallbackSettings {
     pub default_toolchain: Option<String>,
 }
 
-impl Default for FallbackSettings {
-    fn default() -> Self {
-        Self {
-            default_toolchain: None,
-        }
-    }
-}
-
 impl FallbackSettings {
-    pub fn new<P: AsRef<Path>>(path: P) -> Result<Option<Self>> {
+    pub(crate) fn new<P: AsRef<Path>>(path: P) -> Result<Option<Self>> {
         // Users cannot fix issues with missing/unreadable/invalid centralised files, but logging isn't setup early so
         // we can't simply trap all errors and log diagnostics. Ideally we would, and then separate these into different
         // sorts of issues, logging messages about errors that should be fixed. Instead we trap some conservative errors
