@@ -1,21 +1,21 @@
 // Copied from rustc. isatty crate did not work as expected
 #[cfg(unix)]
-pub fn stderr_isatty() -> bool {
+pub(crate) fn stderr_isatty() -> bool {
     isatty(libc::STDERR_FILENO)
 }
 
 #[cfg(windows)]
-pub fn stderr_isatty() -> bool {
+pub(crate) fn stderr_isatty() -> bool {
     isatty(winapi::um::winbase::STD_ERROR_HANDLE)
 }
 
 #[cfg(unix)]
-pub fn stdout_isatty() -> bool {
+pub(crate) fn stdout_isatty() -> bool {
     isatty(libc::STDOUT_FILENO)
 }
 
 #[cfg(windows)]
-pub fn stdout_isatty() -> bool {
+pub(crate) fn stdout_isatty() -> bool {
     isatty(winapi::um::winbase::STD_OUTPUT_HANDLE)
 }
 
